Method: ActionCable::Connection::Base#disconnect
- Defined in:
- lib/action_cable/connection/base.rb
#disconnect ⇒ Object
This method is called every time an Action Cable client disconnects. Override it in your class to cleanup the relevant application state (e.g., presence, online counts, etc.)
88 89 |
# File 'lib/action_cable/connection/base.rb', line 88 def disconnect end |